@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -22,7 +22,9 @@
 
 #define NO_LANGUAGE	-1
 
-int get_word_lang(const char *word, int cur_lang);
+#include "buffer.h"
+
+int check_lang(struct _buffer *p, int cur_lang);
 int get_next_lang(int cur_lang);
 
 #endif /* _DETECTION_H_ */
