@@ -13,10 +13,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
@@ -28,43 +32,45 @@
 #include "xnconfig_memory.h"
 
 #include "types.h"
-#include "utils.h"
 #include "list_char.h"
 #include "log.h"
 
 #include "xnconfig.h"
 
 #define LIBRARY_VERSION_MAJOR		9
-#define LIBRARY_VERSION_MINOR		3
+#define LIBRARY_VERSION_MINOR		5
 #define OPTIONS_DELIMETER		" "
 
 static const char *log_levels[] =	{"Error", "Warning", "Log", "Debug", "Trace"};
 static const char *bool_names[] =	{"No", "Yes"};
+static const char *fix_names[]  =	{"Fixed"};
 static const char *modifier_names[] =	{"Shift", "Control", "Alt", "Super"};
-static const char *flag_names[] =	{"Layout1Flag", "Layout2Flag", "Layout3Flag", "Layout4Flag"};
 
 static const char *option_names[] = 	{
-						"ManualMode", "ExcludeApp", "AddBind", "LogLevel", "AddLanguage", "VowelLetter",
-						"ConsonantLetter", "NoFirstLetter", "SetAutoApp", "SetManualApp", "GrabMouse",
+						"ManualMode", "ExcludeApp", "AddBind", "LogLevel", "AddLanguage", "Autocomplementation",
+						"DisableCapsLock", "CheckOnProcess", "SetAutoApp", "SetManualApp", "AutocomplementationExcludeApp",
 						"EducationMode", "Version", "LayoutRememberMode", "SaveSelectionMode",
 						"DefaultXkbGroup", "AddSound", "PlaySounds", "SendDelay", "LayoutRememberModeForApp",
-						"DrawFlagApp", "AddFlagPixmap", "SaveLog", "ReplaceAbbreviation",
+						"SaveLog", "ReplaceAbbreviation",
 						"ReplaceAbbreviationIgnoreLayout", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
 						"FlushBufferWhenPressEnter", "DontProcessWhenPressEnter", "AddAction",
-						"ShowOSD", "AddOSD", "FontOSD"
+						"ShowOSD", "AddOSD", "FontOSD", "ShowPopup", "AddPopup", 
+	                    "CorrectSpaceWithPunctuation", "AddSpaceAfterAutocomplementation", "LoadModule"
 					};
 static const char *action_names[] =	{
 						"ChangeWord", "ChangeString", "ChangeMode",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected",
+						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard",
 						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
-						"ReplaceAbbreviation"
+						"RotateLayout","ReplaceAbbreviation", "AutocomplementationConfirmation"
 					};
-static const char *sound_names[] =	{
+static const char *notify_names[] =	{
 						"XneurStart", "XneurReload", "XneurStop",
 						"PressKeyLayout1", "PressKeyLayout2", "PressKeyLayout3", "PressKeyLayout4",
 						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
 						"AutomaticChangeWord", "ManualChangeWord", "ChangeString",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected",
+						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard",
 						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
 						"ExecuteUserAction"
 					};
@@ -72,7 +78,6 @@ static const char *sound_names[] =	{
 static int load_lang = -1;
 
 pid_t getsid(pid_t pid);
-char* strsep(char **stringp, const char *delim);
 
 static char* get_word(char **string)
 {
@@ -156,6 +161,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 				break;
 			}
 
+			p->hotkeys[action].key = NULL;
 			while (TRUE)
 			{
 				char *modifier = get_word(&line);
@@ -191,6 +197,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 		{
 			char *dir	= get_word(&line);
 			char *group	= get_word(&line);
+			char *fixed	= get_word(&line);
 
 			if (dir == NULL || group == NULL)
 			{
@@ -198,22 +205,51 @@ static void parse_line(struct _xneur_config *p, char *line)
 				break;
 			}
 
-			p->add_language(p, param, dir, atoi(group));
+			int fix_index = FALSE;
+			if (fixed != NULL)
+			{
+				int index = get_option_index(fix_names, fixed);
+				if (index == 0) // Fixed
+					fix_index = TRUE;
+			}
+
+			p->add_language(p, param, dir, atoi(group), fix_index);
 			break;
 		}
-		case 5: // Get Vowel Letter
+		case 5: // Pattern Mining and Recognition
 		{
-			p->languages[load_lang].vowel_letter = strdup(param);
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for pattern minig and recognition mode specified"));
+				break;
+			}
+
+			p->autocomplementation = index;
 			break;
 		}
-		case 6: // Get Consonant Letter
+		case 6: // Disable CapsLock use
 		{
-			p->languages[load_lang].consonant_letter = strdup(param);
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for disable CapsLock using mode specified"));
+				break;
+			}
+
+			p->disable_capslock = index;
 			break;
 		}
-		case 7: // Get No First Letter
+		case 7: // Check lang on input process
 		{
-			p->languages[load_lang].nofirst_letter = strdup(param);
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for check language on input process mode specified"));
+				break;
+			}
+
+			p->check_lang_on_process = index;
 			break;
 		}
 		case 8: // Get Auto Processing Applications
@@ -226,16 +262,9 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->manual_apps->add(p->manual_apps, full_string);
 			break;
 		}
-		case 10: // Get Mouse Grab Mode
+		case 10: // Autocomplementation Exclude App
 		{
-			int index = get_option_index(bool_names, param);
-			if (index == -1)
-			{
-				log_message(WARNING, _("Invalid value for mouse grab mode specified"));
-				break;
-			}
-
-			p->grab_mouse = index;
+			p->autocomplementation_excluded_apps->add(p->autocomplementation_excluded_apps, full_string);
 			break;
 		}
 		case 11: // Get Education Mode
@@ -286,7 +315,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 		}
 		case 16: // Sounds
 		{
-			int sound = get_option_index(sound_names, param);
+			int sound = get_option_index(notify_names, param);
 			if (sound == -1)
 			{
 				log_message(WARNING, _("Invalid value for sound action name specified"));
@@ -295,7 +324,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 
 			if (line == NULL)
 				break;
-			
+
 			char *file = strdup(get_word(&line));
 			if (strlen(file) != 0)
 			{
@@ -303,7 +332,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			}
 			if (file != NULL)
 				free(file);
-			
+
 			break;
 		}
 		case 17: // Play Sound
@@ -333,32 +362,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->layout_remember_apps->add(p->layout_remember_apps, full_string);
 			break;
 		}
-		case 20: // Get Draw Flag Applications
-		{
-			p->draw_flag_apps->add(p->draw_flag_apps, full_string);
-			break;
-		}
-		case 21: // Flags
-		{
-			int flag = get_option_index(flag_names, param);
-			if (flag == -1)
-			{
-				log_message(WARNING, _("Invalid value for flag layout name specified"));
-				break;
-			}
-
-			if (line == NULL)
-				break;
-			
-			char *file = strdup(get_word(&line));
-			if (strlen(file) != 0)
-			{
-				p->flags[flag].file = get_file_path_name(PIXMAPDIR, file);
-				free(file);
-			}
-			break;
-		}
-		case 22: // Save Keyboard Log
+		case 20: // Save Keyboard Log
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -370,12 +374,12 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->save_keyboard_log = index;
 			break;
 		}
-		case 23: // Get Words for Replacing
+		case 21: // Get Words for Replacing
 		{
 			p->abbreviations->add(p->abbreviations, full_string);
 			break;
 		}
-		case 24: // Ignore keyboard layout for abbreviations
+		case 22: // Ignore keyboard layout for abbreviations
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -387,7 +391,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->abbr_ignore_layout = index;
 			break;
 		}
-		case 25: // Change iNCIDENTAL CapsLock Mode
+		case 23: // Change iNCIDENTAL CapsLock Mode
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -399,7 +403,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->correct_incidental_caps = index;
 			break;
 		}
-		case 26: // Change two CApital letter Mode
+		case 24: // Change two CApital letter Mode
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -411,7 +415,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->correct_two_capital_letter = index;
 			break;
 		}
-		case 27: // Flush internal buffer when pressed Enter Mode
+		case 25: // Flush internal buffer when pressed Enter Mode
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -423,7 +427,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->flush_buffer_when_press_enter = index;
 			break;
 		}
-		case 28: // Don't process word when pressed Enter Mode
+		case 26: // Don't process word when pressed Enter Mode
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -435,40 +439,38 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->dont_process_when_press_enter = index;
 			break;
 		}
-		case 29: // User actions
+		case 27: // User actions
 		{
-			int action = p->actions->action_command->data_count;
-			p->actions->action_hotkey =  (struct _xneur_hotkey *) realloc(p->actions->action_hotkey, (action + 1) * sizeof(struct _xneur_hotkey));
-			p->actions->action_hotkey[action].key = NULL;
-			p->actions->action_hotkey[action].modifiers = 0;
-			
+			p->actions = (struct _xneur_action *) realloc(p->actions, (p->actions_count + 1) * sizeof(struct _xneur_action));
+			bzero(&p->actions[p->actions_count], sizeof(struct _xneur_action));
+
 			while (TRUE)
 			{
-				
 				if (param == NULL)
 					break;
 
 				if (param[0] == '\0')
 					continue;
-				
-				int index = get_option_index(modifier_names, param);
 
+				int index = get_option_index(modifier_names, param);
 				if (index == -1)
 				{
 					if (param != NULL)
-						p->actions->action_hotkey[action].key = strdup(param);
+						p->actions[p->actions_count].hotkey.key = strdup(param);
 					if (line != NULL)
-						p->actions->action_command->add_last(p->actions->action_command, line);
+						p->actions[p->actions_count].command = strdup(line);
 					break;
-				}	
-				else
-					p->actions->action_hotkey[action].modifiers |= (1 << index);
+				}
+
+				p->actions[p->actions_count].hotkey.modifiers |= (1 << index);
 
 				param = get_word(&line);
 			}
+
+			p->actions_count++;
 			break;
 		}
-		case 30: // Show OSD
+		case 28: // Show OSD
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -480,9 +482,9 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->show_osd = index;
 			break;
 		}
-		case 31: // OSDs
+		case 29: // OSDs
 		{
-			int osd = get_option_index(sound_names, param);
+			int osd = get_option_index(notify_names, param);
 			if (osd == -1)
 			{
 				log_message(WARNING, _("Invalid value for OSD action name specified"));
@@ -491,17 +493,69 @@ static void parse_line(struct _xneur_config *p, char *line)
 
 			if (line == NULL)
 				break;
-			
+
 			if (strlen(line) != 0)
 				p->osds[osd].file = strdup(line);
-			
+
 			break;
 		}
-		case 32: // Get Initial Xkb Group for all new windows
+		case 30: // Get Initial Xkb Group for all new windows
 		{
 			p->osd_font = strdup(param);
 			break;
 		}
+		case 31: // Show popup
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for show popup message mode specified"));
+				break;
+			}
+
+			p->show_popup = index;
+			break;
+		}
+		case 32: // Popups
+		{
+			int popup = get_option_index(notify_names, param);
+			if (popup == -1)
+			{
+				log_message(WARNING, _("Invalid value for OSD action name specified"));
+				break;
+			}
+
+			if (line == NULL)
+				break;
+
+			if (strlen(line) != 0)
+				p->popups[popup].file = strdup(line);
+
+			break;
+		}
+		case 33:
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+				break;
+
+			p->correct_space_with_punctuation = index;
+			break;
+		}
+		case 34:
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+				break;
+
+			p->add_space_after_autocomplementation = index;
+			break;
+		}
+		case 35: // Add plugin 
+		{
+			p->plugins->add(p->plugins, full_string);
+			break;
+		}
 	}
 	free(full_string);
 }
@@ -536,95 +590,82 @@ static int check_memory_attached(struct _xneur_config *p)
 
 static void free_structures(struct _xneur_config *p)
 {
-	int total_user_actions = p->actions->action_command->data_count;
-	
 	p->window_layouts->uninit(p->window_layouts);
 	p->manual_apps->uninit(p->manual_apps);
 	p->auto_apps->uninit(p->auto_apps);
 	p->layout_remember_apps->uninit(p->layout_remember_apps);
 	p->excluded_apps->uninit(p->excluded_apps);
-	p->draw_flag_apps->uninit(p->draw_flag_apps);
-	p->actions->action_command->uninit(p->actions->action_command);
+	p->autocomplementation_excluded_apps->uninit(p->autocomplementation_excluded_apps);
 	
-	if (p->version != NULL)
+	for (int hotkey = 0; hotkey < MAX_HOTKEYS; hotkey++)
 	{
-		free(p->version);
-		p->version = NULL;
+		if (p->hotkeys[hotkey].key != NULL)
+			free(p->hotkeys[hotkey].key);
+	}
+
+	for (int notify = 0; notify < MAX_NOTIFIES; notify++)
+	{
+		if (p->sounds[notify].file != NULL)
+			free(p->sounds[notify].file);
+
+		if (p->osds[notify].file != NULL)
+			free(p->osds[notify].file);
+
+		if (p->popups[notify].file != NULL)
+			free(p->popups[notify].file);
 	}
 
 	for (int lang = 0; lang < p->total_languages; lang++)
 	{
-		if (p->languages[lang].temp_dicts != NULL)
-			p->languages[lang].temp_dicts->uninit(p->languages[lang].temp_dicts);
+		if (p->languages[lang].temp_dict != NULL)
+			p->languages[lang].temp_dict->uninit(p->languages[lang].temp_dict);
 
-		if (p->languages[lang].dicts != NULL)
-			p->languages[lang].dicts->uninit(p->languages[lang].dicts);
+		if (p->languages[lang].dict != NULL)
+			p->languages[lang].dict->uninit(p->languages[lang].dict);
 
-		if (p->languages[lang].protos != NULL)
-			p->languages[lang].protos->uninit(p->languages[lang].protos);
+		if (p->languages[lang].proto != NULL)
+			p->languages[lang].proto->uninit(p->languages[lang].proto);
 
-		if (p->languages[lang].big_protos != NULL)
-			p->languages[lang].big_protos->uninit(p->languages[lang].big_protos);
+		if (p->languages[lang].big_proto != NULL)
+			p->languages[lang].big_proto->uninit(p->languages[lang].big_proto);
 
 		if (p->languages[lang].regexp != NULL)
 			p->languages[lang].regexp->uninit(p->languages[lang].regexp);
 
-		if (p->languages[lang].vowel_letter != NULL)
-			free(p->languages[lang].vowel_letter);
-
-		if (p->languages[lang].consonant_letter != NULL)
-			free(p->languages[lang].consonant_letter);
-
-		if (p->languages[lang].nofirst_letter != NULL)
-			free(p->languages[lang].nofirst_letter);
-
+		if (p->languages[lang].pattern != NULL)
+			p->languages[lang].pattern->uninit(p->languages[lang].pattern);
+		
 		free(p->languages[lang].name);
 		free(p->languages[lang].dir);
 	}
 
+	for (int action = 0; action < p->actions_count; action++)
+	{
+		if (p->actions[action].hotkey.key != NULL)
+			free(p->actions[action].hotkey.key);
+		if (p->actions[action].command != NULL)
+			free(p->actions[action].command);
+	}
+
+	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
+	bzero(p->sounds, MAX_NOTIFIES * sizeof(struct _xneur_file));
+	bzero(p->osds, MAX_NOTIFIES * sizeof(struct _xneur_file));
+	bzero(p->popups, MAX_NOTIFIES * sizeof(struct _xneur_file));
+
 	p->total_languages = 0;
+	p->actions_count = 0;
+
+	if (p->version != NULL)
+		free(p->version);
+
+	if (p->osd_font != NULL)
+		free(p->osd_font);
 
 	if (p->languages != NULL)
-	{
 		free(p->languages);
-		p->languages = NULL;
-	}
 
-	for (int user_action = 0; user_action < total_user_actions; user_action++)
-	{
-		if (p->actions->action_hotkey[user_action].key != NULL)
-			free(p->actions->action_hotkey[user_action].key);
-	}
-		
-	for (int action = 0; action < MAX_HOTKEYS; action++)
-	{
-		if (p->hotkeys[action].key != NULL)
-			free(p->hotkeys[action].key);
-	}
-
-	for (int sound = 0; sound < MAX_SOUNDS; sound++)
-	{
-		if (p->sounds[sound].file != NULL)
-			free(p->sounds[sound].file);
-	}
-
-	for (int osd = 0; osd < MAX_OSDS; osd++)
-	{
-		if (p->osds[osd].file != NULL)
-			free(p->osds[osd].file);
-	}
-	
-	for (int flag = 0; flag < MAX_FLAGS; flag++)
-	{
-		if (p->flags[flag].file != NULL)
-			free(p->flags[flag].file);
-	}
-
-	bzero(p->actions->action_hotkey, total_user_actions * sizeof(struct _xneur_hotkey));
-	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
-	bzero(p->sounds, MAX_SOUNDS * sizeof(struct _xneur_file));
-	bzero(p->osds, MAX_OSDS * sizeof(struct _xneur_file));
-	bzero(p->flags, MAX_FLAGS * sizeof(struct _xneur_file));
+	if (p->actions != NULL)
+		free(p->actions);
 }
 
 static void xneur_config_reload(struct _xneur_config *p)
@@ -693,24 +734,30 @@ static int xneur_config_load(struct _xneur_config *p)
 		char *lang_dir	= p->get_lang_dir(p, lang);
 		char *lang_name	= p->get_lang_name(p, lang);
 
-		p->languages[lang].dicts = load_list(lang_dir, DICT_NAME, TRUE);
-		if (p->languages[lang].dicts == NULL)
+		p->languages[lang].dict = load_list(lang_dir, DICT_NAME, TRUE);
+		if (p->languages[lang].dict == NULL)
 		{
 			log_message(ERROR, _("Can't find dictionary file for %s language"), lang_name);
+			if (lang_dir != NULL)
+				free(lang_dir);
 			return FALSE;
 		}
 
-		p->languages[lang].protos = load_list(lang_dir, PROTO_NAME, TRUE);
-		if (p->languages[lang].protos == NULL)
+		p->languages[lang].proto = load_list(lang_dir, PROTO_NAME, TRUE);
+		if (p->languages[lang].proto == NULL)
 		{
 			log_message(ERROR, _("Can't find protos file for %s language"), lang_name);
+			if (lang_dir != NULL)
+				free(lang_dir);
 			return FALSE;
 		}
 
-		p->languages[lang].big_protos = load_list(lang_dir, BIG_PROTO_NAME, TRUE);
-		if (p->languages[lang].big_protos == NULL)
+		p->languages[lang].big_proto = load_list(lang_dir, BIG_PROTO_NAME, TRUE);
+		if (p->languages[lang].big_proto == NULL)
 		{
 			log_message(ERROR, _("Can't find big protos file for %s language"), lang_name);
+			if (lang_dir != NULL)
+				free(lang_dir);
 			return FALSE;
 		}
 
@@ -718,16 +765,25 @@ static int xneur_config_load(struct _xneur_config *p)
 		if (p->languages[lang].regexp == NULL)
 		{
 			log_message(ERROR, _("Can't find regexp file for %s language"), lang_name);
+			if (lang_dir != NULL)
+				free(lang_dir);
 			return FALSE;
 		}
 
-		p->languages[lang].temp_dicts = p->languages[lang].dicts->clone(p->languages[lang].dicts);
+		p->languages[lang].pattern = load_list(lang_dir, PATTERN_NAME, TRUE);
+		if (p->languages[lang].pattern == NULL)
+		{
+			log_message(WARNING, _("Can't find pattern file for %s language"), lang_name);
+		}
+		
+		p->languages[lang].temp_dict = p->languages[lang].dict->clone(p->languages[lang].dict);
 
 		load_lang = lang;
-		if (!parse_config_file(p, lang_dir, LANGDEF_NAME))
-			return FALSE;
-	}
 
+		if (lang_dir != NULL)
+			free(lang_dir);
+	}
+	
 	return TRUE;
 }
 
@@ -735,14 +791,19 @@ static void xneur_config_clear(struct _xneur_config *p)
 {
 	free_structures(p);
 
-	p->window_layouts	= list_char_init();
-	p->manual_apps		= list_char_init();
-	p->auto_apps		= list_char_init();
-	p->layout_remember_apps	= list_char_init();
-	p->excluded_apps	= list_char_init();
-	p->draw_flag_apps	= list_char_init();
-	p->abbreviations	= list_char_init();
-	p->actions->action_command = list_char_init();
+	p->window_layouts		= list_char_init();
+	p->manual_apps			= list_char_init();
+	p->auto_apps			= list_char_init();
+	p->layout_remember_apps		= list_char_init();
+	p->excluded_apps		= list_char_init();
+	p->autocomplementation_excluded_apps	= list_char_init();
+	p->abbreviations		= list_char_init();
+	p->plugins				= list_char_init();
+	
+	p->version	= NULL;
+	p->osd_font	= NULL;
+	p->languages	= NULL;
+	p->actions	= NULL;
 }
 
 static int xneur_config_save(struct _xneur_config *p)
@@ -778,7 +839,13 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# See Settings page on http://www.xneur.ru for details\n");
 
 	for (int lang = 0; lang < p->total_languages; lang++)
-		fprintf(stream, "AddLanguage %s %s %d\n", p->languages[lang].name, p->languages[lang].dir, p->languages[lang].group);
+	{
+		fprintf(stream, "AddLanguage %s %s %d ", p->languages[lang].name, p->languages[lang].dir, p->languages[lang].group);
+		if (p->languages[lang].fixed)
+			fprintf(stream, "%s\n", fix_names[0]);
+		else
+			fprintf(stream, "\n");
+	}
 	fprintf(stream, "\n");
 
 	fprintf(stream, "# Define initial keyboard layout for all new applications\n");
@@ -827,27 +894,26 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# This option add user action when pressed key bind\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#AddAction Control Alt f firefox\n");
-	for (int action = 0; action < p->actions->action_command->data_count; action++)
+	for (int action = 0; action < p->actions_count; action++)
 	{
 		fprintf(stream, "AddAction ");
 
 		const int total_modifiers = sizeof(modifier_names) / sizeof(modifier_names[0]);
 		for (int i = 0; i < total_modifiers; i++)
 		{
-			if (p->actions->action_hotkey[action].modifiers & (1 << i))
+			if (p->actions[action].hotkey.modifiers & (1 << i))
 				fprintf(stream, "%s ", modifier_names[i]);
 		}
 
-		fprintf(stream, "%s %s\n", p->actions->action_hotkey[action].key
-								, p->actions->action_command->data[action].string);
+		fprintf(stream, "%s %s\n", p->actions[action].hotkey.key, p->actions[action].command);
 	}
 	fprintf(stream, "\n");
-			
+
 	fprintf(stream, "# Word Replacing\n# Ignore keyboard layout for abbreviations list\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#ReplaceAbbreviationIgnoreLayout No\n");
 	fprintf(stream, "ReplaceAbbreviationIgnoreLayout %s\n\n", p->get_bool_name(p->abbr_ignore_layout));
-			
+
 	fprintf(stream, "# Abbreviations list\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#ReplaceAbbreviation xneur X Neural Switcher\n");
@@ -861,20 +927,15 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "PlaySounds %s\n\n", p->get_bool_name(p->play_sounds));
 
 	fprintf(stream, "# Binds sounds for some actions\n");
-	for (int sound = 0; sound < MAX_SOUNDS; sound++)
+	for (int sound = 0; sound < MAX_NOTIFIES; sound++)
 	{
 		if (p->sounds[sound].file == NULL)
-			fprintf(stream, "AddSound %s \n", sound_names[sound]);
+			fprintf(stream, "AddSound %s \n", notify_names[sound]);
 		else
-			fprintf(stream, "AddSound %s %s\n", sound_names[sound], p->sounds[sound].file);
+			fprintf(stream, "AddSound %s %s\n", notify_names[sound], p->sounds[sound].file);
 	}
 	fprintf(stream, "\n");
 
-	fprintf(stream, "# This option enable or disable mouse processing\n");
-	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#GrabMouse Yes\n");
-	fprintf(stream, "GrabMouse %s\n\n", p->get_bool_name(p->grab_mouse));
-
 	fprintf(stream, "# This option enable or disable self education of xneur\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#EducationMode No\n");
@@ -901,25 +962,6 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# This option define delay before sendind events to application (in milliseconds between 0 to 50).\n");
 	fprintf(stream, "SendDelay %d\n\n", p->send_delay);
 
-	fprintf(stream, "# Binds pixmaps for some layouts\n");
-	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#AddFlagPixmap <Layout1Flag|Layout2Flag|Layout3Flag|Layout4Flag> English.png\n");
-	for (int flag = 0; flag < MAX_FLAGS; flag++)
-	{
-		if (p->flags[flag].file == NULL)
-			continue;
-
-		fprintf(stream, "AddFlagPixmap %s %s\n", flag_names[flag], p->flags[flag].file);
-	}
-	fprintf(stream, "\n");
-
-	fprintf(stream, "# Add Applications names to draw flag in window\n");
-	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#DrawFlagApp Gedit\n");
-	for (int i = 0; i < p->draw_flag_apps->data_count; i++)
-		fprintf(stream, "DrawFlagApp %s\n", p->draw_flag_apps->data[i].string);
-	fprintf(stream, "\n");
-
 	fprintf(stream, "# This option enable or disable logging keyboard\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#SaveLog No\n");
@@ -934,7 +976,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#CorrectTwoCapitalLetter Yes\n");
 	fprintf(stream, "CorrectTwoCapitalLetter %s\n\n", p->get_bool_name(p->correct_two_capital_letter));
-			
+
 	fprintf(stream, "# This option enable or disable flushing internal buffer when pressed Enter or Tab\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#FlushBufferWhenPressEnter Yes\n");
@@ -944,7 +986,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#DontProcessWhenPressEnter Yes\n");
 	fprintf(stream, "DontProcessWhenPressEnter %s\n\n", p->get_bool_name(p->dont_process_when_press_enter));
-			
+
 	fprintf(stream, "# This option disable or enable show OSD\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#ShowOSD Yes\n");
@@ -954,17 +996,71 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#FontOSD -*-*-*-*-*-*-32-*-*-*-*-*-*-u\n");
 	fprintf(stream, "FontOSD %s\n\n", p->osd_font);
-			
+
 	fprintf(stream, "# Binds OSDs for some actions\n");
-	for (int osd= 0; osd < MAX_OSDS; osd++)
+	for (int notify = 0; notify < MAX_NOTIFIES; notify++)
 	{
-		if (p->osds[osd].file == NULL)
-			fprintf(stream, "AddOSD %s \n", sound_names[osd]);
+		if (p->osds[notify].file == NULL)
+			fprintf(stream, "AddOSD %s\n", notify_names[notify]);
 		else
-			fprintf(stream, "AddOSD %s %s\n", sound_names[osd], p->osds[osd].file);
+			fprintf(stream, "AddOSD %s %s\n", notify_names[notify], p->osds[notify].file);
 	}
+
+	fprintf(stream, "\n# This option disable or enable show popup messages\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#ShowPopup Yes\n");
+	fprintf(stream, "ShowPopup %s\n\n", p->get_bool_name(p->show_popup));
+
+	fprintf(stream, "# Binds popup messages for some actions\n");
+	for (int notify = 0; notify < MAX_NOTIFIES; notify++)
+	{
+		if (p->popups[notify].file == NULL)
+			fprintf(stream, "AddPopup %s\n", notify_names[notify]);
+		else
+			fprintf(stream, "AddPopup %s %s\n", notify_names[notify], p->popups[notify].file);
+	}
+
+	fprintf(stream, "\n# This option disable or enable checking language on input process\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CheckOnProcess Yes\n");
+	fprintf(stream, "CheckOnProcess %s\n", p->get_bool_name(p->check_lang_on_process));
+
+	fprintf(stream, "\n# This option disable or enable CapsLock use\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#DisableCapsLock Yes\n");
+	fprintf(stream, "DisableCapsLock %s\n", p->get_bool_name(p->disable_capslock));
+
+	fprintf(stream, "\n# This option disable or enable correction spaces befor punctuation\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectSpaceWithPunctuation No\n");
+	fprintf(stream, "CorrectSpaceWithPunctuation %s\n", p->get_bool_name(p->correct_space_with_punctuation));
+
+	fprintf(stream, "\n# This option disable or enable pattern mining and recognition (autocomplementation)\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#Autocomplementation No\n");
+	fprintf(stream, "Autocomplementation %s\n", p->get_bool_name(p->autocomplementation));
+
+	fprintf(stream, "\n# This option disable or enable adding space after autocomplementation\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#AddSpaceAfterAutocomplementation No\n");
+	fprintf(stream, "AddSpaceAfterAutocomplementation %s\n", p->get_bool_name(p->add_space_after_autocomplementation));
 	fprintf(stream, "\n");
 
+	fprintf(stream, "# Add Applications names to exclude it from autocomplementation process\n");
+	fprintf(stream, "# Xneur will not process the autocomplementation for this applications\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#AutocomplementationExcludeApp Gnome-terminal\n");
+	for (int i = 0; i < p->autocomplementation_excluded_apps->data_count; i++)
+		fprintf(stream, "AutocomplementationExcludeApp %s\n", p->autocomplementation_excluded_apps->data[i].string);
+	fprintf(stream, "\n");
+
+	fprintf(stream, "# Modules list\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#LoadModule libxntest.so\n");
+	for (int plugins = 0; plugins < p->plugins->data_count; plugins++)
+		fprintf(stream, "LoadModule %s\n", p->plugins->data[plugins].string);
+	fprintf(stream, "\n");
+	        
 	fprintf(stream, "# That's all\n");
 
 	fclose(stream);
@@ -996,21 +1092,40 @@ static int xneur_config_replace(struct _xneur_config *p)
 	return p->load(p);
 }
 
-static void xneur_config_save_dicts(struct _xneur_config *p, int lang)
+static void xneur_config_save_dict(struct _xneur_config *p, int lang)
 {
 	if (!p->educate)
 		return;
 
 	log_message(LOG, _("Saving %s dictionary"), p->get_lang_name(p, lang));
 
-	save_list(p->languages[lang].dicts, p->get_lang_dir(p, lang), DICT_NAME);
+	char *lang_dir = p->get_lang_dir(p, lang);
+	save_list(p->languages[lang].dict, lang_dir, DICT_NAME);
+	free(lang_dir);
+}
+
+static void xneur_config_save_pattern(struct _xneur_config *p, int lang)
+{
+	if (!p->educate)
+		return;
+
+	log_message(LOG, _("Saving %s pattern"), p->get_lang_name(p, lang));
+
+	char *lang_dir = p->get_lang_dir(p, lang);
+	save_list(p->languages[lang].pattern, lang_dir, PATTERN_NAME);
+	free(lang_dir);
 }
 
 static char* xneur_config_get_lang_dir(struct _xneur_config *p, int lang)
 {
 	if (lang < 0 || lang >= p->total_languages)
 		return NULL;
-	return p->languages[lang].dir;
+
+	int path_len = strlen(LANGUAGEDIR) + strlen(p->languages[lang].dir) + 2;
+	char *path_file = (char *) malloc(path_len * sizeof(char));
+	snprintf(path_file, path_len, "%s/%s", LANGUAGEDIR, p->languages[lang].dir);
+
+	return path_file;
 }
 
 static char* xneur_config_get_lang_name(struct _xneur_config *p, int lang)
@@ -1037,7 +1152,7 @@ static int xneur_config_find_group_lang(struct _xneur_config *p, int group)
 	return -1;
 }
 
-static void xneur_config_add_language(struct _xneur_config *p, const char *name, const char *dir, int group)
+static void xneur_config_add_language(struct _xneur_config *p, const char *name, const char *dir, int group, int fixed)
 {
 	if (name == NULL || dir == NULL)
 	{
@@ -1051,7 +1166,7 @@ static void xneur_config_add_language(struct _xneur_config *p, const char *name,
 	p->languages[p->total_languages].name	= strdup(name);
 	p->languages[p->total_languages].dir	= strdup(dir);
 	p->languages[p->total_languages].group	= group;
-
+	p->languages[p->total_languages].fixed	= fixed;
 	p->total_languages++;
 }
 
@@ -1063,14 +1178,12 @@ static const char* xneur_config_get_log_level_name(struct _xneur_config *p)
 static void xneur_config_uninit(struct _xneur_config *p)
 {
 	free_structures(p);
-	
+
 	free(p->hotkeys);
 	free(p->sounds);
 	free(p->osds);
-	free(p->flags);
-	
-	free(p->actions);
-	
+	free(p->popups);
+
 	free(p);
 }
 
@@ -1088,14 +1201,14 @@ struct _xneur_config* xneur_config_init(void)
 	p->hotkeys = (struct _xneur_hotkey *) malloc(MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
 	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
 
-	p->sounds = (struct _xneur_file *) malloc(MAX_SOUNDS * sizeof(struct _xneur_file));
-	bzero(p->sounds, MAX_SOUNDS * sizeof(struct _xneur_file));
+	p->sounds = (struct _xneur_file *) malloc(MAX_NOTIFIES * sizeof(struct _xneur_file));
+	bzero(p->sounds, MAX_NOTIFIES * sizeof(struct _xneur_file));
 
-	p->osds = (struct _xneur_file *) malloc(MAX_OSDS * sizeof(struct _xneur_file));
-	bzero(p->osds, MAX_OSDS * sizeof(struct _xneur_file));
-	
-	p->flags = (struct _xneur_file *) malloc(MAX_FLAGS * sizeof(struct _xneur_file));
-	bzero(p->flags, MAX_FLAGS * sizeof(struct _xneur_file));
+	p->osds = (struct _xneur_file *) malloc(MAX_NOTIFIES * sizeof(struct _xneur_file));
+	bzero(p->osds, MAX_NOTIFIES * sizeof(struct _xneur_file));
+
+	p->popups = (struct _xneur_file *) malloc(MAX_NOTIFIES * sizeof(struct _xneur_file));
+	bzero(p->popups, MAX_NOTIFIES * sizeof(struct _xneur_file));
 
 	p->log_level			= LOG;
 	p->excluded_apps		= list_char_init();
@@ -1103,14 +1216,10 @@ struct _xneur_config* xneur_config_init(void)
 	p->manual_apps			= list_char_init();
 	p->layout_remember_apps		= list_char_init();
 	p->window_layouts		= list_char_init();
-	p->draw_flag_apps		= list_char_init();
 	p->abbreviations		= list_char_init();
+	p->autocomplementation_excluded_apps	= list_char_init();
+	p->plugins		= list_char_init();
 
-	p->actions = (struct _xneur_action *) malloc(sizeof(struct _xneur_action));
-	p->actions->action_hotkey = (struct _xneur_hotkey *) malloc(sizeof(struct _xneur_hotkey));
-	bzero(p->actions->action_hotkey, sizeof(struct _xneur_hotkey));
-	p->actions->action_command	= list_char_init();
-	
 	// Function mapping
 	p->get_home_dict_path		= get_home_file_path_name;
 	p->get_global_dict_path		= get_file_path_name;
@@ -1124,7 +1233,8 @@ struct _xneur_config* xneur_config_init(void)
 	p->replace			= xneur_config_replace;
 	p->reload			= xneur_config_reload;
 	p->kill				= xneur_config_kill;
-	p->save_dicts			= xneur_config_save_dicts;
+	p->save_dict			= xneur_config_save_dict;
+	p->save_pattern			= xneur_config_save_pattern;
 	p->set_manual_mode		= xneur_config_set_manual_mode;
 	p->is_manual_mode		= xneur_config_is_manual_mode;
 	p->set_pid			= xneur_config_set_pid;
