@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -27,16 +27,15 @@
 #define DICT_NAME			"dict"
 #define PROTO_NAME			"proto"
 #define BIG_PROTO_NAME			"proto3"
-#define LANGDEF_NAME			"langdef"
 #define REGEXP_NAME			"regexp"
+#define PATTERN_NAME		"pattern"
 
 #define SOUNDDIR			"sounds"
-#define PIXMAPDIR			"pixmaps"
+#define LANGUAGEDIR			"languages"
 
 #define MAX_FLAGS			4
-#define MAX_SOUNDS			21
-#define MAX_OSDS			21
-#define MAX_HOTKEYS			11
+#define MAX_NOTIFIES		24
+#define MAX_HOTKEYS			16
 
 enum _flag_action
 {
@@ -46,56 +45,33 @@ enum _flag_action
 	FLAG_LAYOUT_3,
 };
 
-enum _sound_action
+enum _notify_action
 {
-	SOUND_XNEUR_START = 0,
-	SOUND_XNEUR_RELOAD,
-	SOUND_XNEUR_STOP,
-	SOUND_PRESS_KEY_LAYOUT_0,
-	SOUND_PRESS_KEY_LAYOUT_1,
-	SOUND_PRESS_KEY_LAYOUT_2,
-	SOUND_PRESS_KEY_LAYOUT_3,
-	SOUND_ENABLE_LAYOUT_0,
-	SOUND_ENABLE_LAYOUT_1,
-	SOUND_ENABLE_LAYOUT_2,
-	SOUND_ENABLE_LAYOUT_3,
-	SOUND_AUTOMATIC_CHANGE_WORD,
-	SOUND_MANUAL_CHANGE_WORD,
-	SOUND_CHANGE_STRING,
-	SOUND_CHANGE_SELECTED,
-	SOUND_TRANSLIT_SELECTED,
-	SOUND_CHANGECASE_SELECTED,
-	SOUND_REPLACE_ABBREVIATION,
-	SOUND_CORR_INCIDENTAL_CAPS,
-	SOUND_CORR_TWO_CAPITAL_LETTER,
-	SOUND_EXEC_USER_ACTION,
-	SOUND_NONE,
-};
-
-enum _osd_action
-{
-	OSD_XNEUR_START = 0,
-	OSD_XNEUR_RELOAD,
-	OSD_XNEUR_STOP,
-	OSD_PRESS_KEY_LAYOUT_0,
-	OSD_PRESS_KEY_LAYOUT_1,
-	OSD_PRESS_KEY_LAYOUT_2,
-	OSD_PRESS_KEY_LAYOUT_3,
-	OSD_ENABLE_LAYOUT_0,
-	OSD_ENABLE_LAYOUT_1,
-	OSD_ENABLE_LAYOUT_2,
-	OSD_ENABLE_LAYOUT_3,
-	OSD_AUTOMATIC_CHANGE_WORD,
-	OSD_MANUAL_CHANGE_WORD,
-	OSD_CHANGE_STRING,
-	OSD_CHANGE_SELECTED,
-	OSD_TRANSLIT_SELECTED,
-	OSD_CHANGECASE_SELECTED,
-	OSD_REPLACE_ABBREVIATION,
-	OSD_CORR_INCIDENTAL_CAPS,
-	OSD_CORR_TWO_CAPITAL_LETTER,
-	OSD_EXEC_USER_ACTION,
-	OSD_NONE,
+	NOTIFY_XNEUR_START = 0,
+	NOTIFY_XNEUR_RELOAD,
+	NOTIFY_XNEUR_STOP,
+	NOTIFY_PRESS_KEY_LAYOUT_0,
+	NOTIFY_PRESS_KEY_LAYOUT_1,
+	NOTIFY_PRESS_KEY_LAYOUT_2,
+	NOTIFY_PRESS_KEY_LAYOUT_3,
+	NOTIFY_ENABLE_LAYOUT_0,
+	NOTIFY_ENABLE_LAYOUT_1,
+	NOTIFY_ENABLE_LAYOUT_2,
+	NOTIFY_ENABLE_LAYOUT_3,
+	NOTIFY_AUTOMATIC_CHANGE_WORD,
+	NOTIFY_MANUAL_CHANGE_WORD,
+	NOTIFY_CHANGE_STRING,
+	NOTIFY_CHANGE_SELECTED,
+	NOTIFY_TRANSLIT_SELECTED,
+	NOTIFY_CHANGECASE_SELECTED,
+	NOTIFY_CHANGE_CLIPBOARD,
+	NOTIFY_TRANSLIT_CLIPBOARD,
+	NOTIFY_CHANGECASE_CLIPBOARD,
+	NOTIFY_REPLACE_ABBREVIATION,
+	NOTIFY_CORR_INCIDENTAL_CAPS,
+	NOTIFY_CORR_TWO_CAPITAL_LETTER,
+	NOTIFY_EXEC_USER_ACTION,
+	NOTIFY_NONE,
 };
 
 enum _hotkey_action
@@ -106,22 +82,37 @@ enum _hotkey_action
 	ACTION_CHANGE_SELECTED,
 	ACTION_TRANSLIT_SELECTED,
 	ACTION_CHANGECASE_SELECTED,
+	ACTION_CHANGE_CLIPBOARD,
+	ACTION_TRANSLIT_CLIPBOARD,
+	ACTION_CHANGECASE_CLIPBOARD,
 	ACTION_ENABLE_LAYOUT_0,
 	ACTION_ENABLE_LAYOUT_1,
 	ACTION_ENABLE_LAYOUT_2,
 	ACTION_ENABLE_LAYOUT_3,
+	ACTION_ROTATE_LAYOUT,
 	ACTION_REPLACE_ABBREVIATION,
+	ACTION_AUTOCOMPLEMENTATION,
 	ACTION_NONE,
 };
 
 enum _change_action
 {
-	CORR_INCIDENTAL_CAPS = -2,
-	CORR_TWO_CAPITAL_LETTER,
-	ENABLE_LAYOUT_0,
-	ENABLE_LAYOUT_1,
-	ENABLE_LAYOUT_2,
-	ENABLE_LAYOUT_3,
+	CHANGE_INCIDENTAL_CAPS = 0,
+	CHANGE_TWO_CAPITAL_LETTER,
+	CHANGE_WORD_TO_LAYOUT_0,
+	CHANGE_WORD_TO_LAYOUT_1,
+	CHANGE_WORD_TO_LAYOUT_2,
+	CHANGE_WORD_TO_LAYOUT_3,
+	CHANGE_SYLL_TO_LAYOUT_0,
+	CHANGE_SYLL_TO_LAYOUT_1,
+	CHANGE_SYLL_TO_LAYOUT_2,
+	CHANGE_SYLL_TO_LAYOUT_3,
+	CHANGE_SELECTION,
+	CHANGE_STRING_TO_LAYOUT_0,
+	CHANGE_STRING_TO_LAYOUT_1,
+	CHANGE_STRING_TO_LAYOUT_2,
+	CHANGE_STRING_TO_LAYOUT_3,
+	CHANGE_ABBREVIATION,
 };
 
 struct _xneur_language
@@ -129,16 +120,14 @@ struct _xneur_language
 	char *dir;
 	char *name;
 	int  group;
+	int  fixed;
 
-	char *vowel_letter;
-	char *consonant_letter;
-	char *nofirst_letter;
-
-	struct _list_char *temp_dicts;
-	struct _list_char *dicts;
-	struct _list_char *protos;
-	struct _list_char *big_protos;
+	struct _list_char *temp_dict;
+	struct _list_char *dict;
+	struct _list_char *proto;
+	struct _list_char *big_proto;
 	struct _list_char *regexp;
+	struct _list_char *pattern;
 };
 
 struct _xneur_hotkey
@@ -160,8 +149,8 @@ struct _xneur_data
 
 struct _xneur_action
 {
-	struct _xneur_hotkey *action_hotkey;
-	struct _list_char *action_command;
+	struct _xneur_hotkey hotkey;
+	char *command;
 };
 
 struct _xneur_config
@@ -175,12 +164,18 @@ struct _xneur_config
 	struct _list_char *manual_apps;
 	struct _list_char *layout_remember_apps;
 	struct _list_char *window_layouts;
-
+	struct _list_char *abbreviations;
+	struct _list_char *plugins;
+	
 	struct _xneur_data *xneur_data;
 	struct _xneur_language *languages;		// Array of languages used in program
 	struct _xneur_hotkey *hotkeys;			// Array of hotkeys used in program
 	struct _xneur_file *sounds;			// Array of sounds for actions
 	struct _xneur_file *osds;			// Array of OSDs for actions
+	struct _xneur_file *popups;			// Array of popups for actions
+	struct _xneur_action *actions;			// Array of actions
+
+	int   actions_count;				// Count of actions
 
 	int   manual_mode;				// Enable manual processing mode
 	int   log_level;				// Maximum level of log messages to print
@@ -190,28 +185,29 @@ struct _xneur_config
 	int   default_group;				// Initial keyboard layout for all new applications
 
 	int   play_sounds;				// Play sound samples or not
-	int   grab_mouse;				// Grab mouse or not
 	int   educate;					// Education xneur
 	int   remember_layout;				// Remember layout for each of window
 	int   save_selection;				// Save selection after convert
 	int   save_keyboard_log;			// Save keyboard log
-	int   correct_incidental_caps;		// Change iNCIDENTAL CapsLock
-	int   correct_two_capital_letter;	// Change two CApital letter
-	int   flush_buffer_when_press_enter;  // Flush internal buffer when pressed Enter
-	int   dont_process_when_press_enter;   // Don't correct word when pressed Enter
-
-	int   show_osd;					// Show OSD 
+	int   correct_incidental_caps;			// Change iNCIDENTAL CapsLock
+	int   correct_two_capital_letter;		// Change two CApital letter
+	int   correct_space_with_punctuation;	// Correct spaces before punctuation
+	int   flush_buffer_when_press_enter;		// Flush internal buffer when pressed Enter
+	int   dont_process_when_press_enter;		// Don't correct word when pressed Enter
+	int   check_lang_on_process;			// Check lang on input process
+	int   disable_capslock;				// Disable CapsLock use
+	
+	int   autocomplementation;			// Save pattern and mining
+	int   add_space_after_autocomplementation;
+	struct _list_char *autocomplementation_excluded_apps;
+	
+	int   show_osd;					// Show OSD
 	char  *osd_font;
 
+	int   show_popup;				// Show popups
+
 	int   abbr_ignore_layout;			// Ignore keyboard layout for abbreviations
 	
-	struct _list_char *draw_flag_apps;
-	struct _xneur_file *flags;			// Array of flag pixmaps for draw
-
-	struct _list_char *abbreviations;		// Array of words to replace
-
-	struct _xneur_action *actions;
-
 	char* (*get_home_dict_path) (const char *dir_name, const char *file_name);
 	char* (*get_global_dict_path) (const char *dir_name, const char *file_name);
 	const char* (*get_bool_name) (int option);
@@ -222,7 +218,8 @@ struct _xneur_config
 	int   (*replace) (struct _xneur_config *p);
 	void  (*reload) (struct _xneur_config *p);
 	int   (*kill) (struct _xneur_config *p);
-	void  (*save_dicts) (struct _xneur_config *p, int lang);
+	void  (*save_dict) (struct _xneur_config *p, int lang);
+	void  (*save_pattern) (struct _xneur_config *p, int lang);
 	void  (*set_pid) (struct _xneur_config *p, int pid);
 	int   (*get_pid) (struct _xneur_config *p);
 	void  (*set_manual_mode) (struct _xneur_config *p, int manual_mode);
@@ -231,7 +228,7 @@ struct _xneur_config
 	char* (*get_lang_name) (struct _xneur_config *p, int lang);
 	int   (*get_lang_group) (struct _xneur_config *p, int lang);
 	int   (*find_group_lang) (struct _xneur_config *p, int group);
-	void  (*add_language) (struct _xneur_config *p, const char *name, const char *dir, int group);
+	void  (*add_language) (struct _xneur_config *p, const char *name, const char *dir, int group, int fixed);
 	const char* (*get_log_level_name) (struct _xneur_config *p);
 	void  (*uninit) (struct _xneur_config *p);
 };
