@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -31,6 +31,7 @@
 
 #define HOME_CONF_DIR		".xneur"
 #define DEFAULT_MAX_PATH	4096
+#define DIR_SEPARATOR		"/"
 
 static int get_max_path_len(void)
 {
@@ -52,7 +53,7 @@ static char* get_file_content(const char *file_name)
 		return NULL;
 
 	unsigned int file_len = sb.st_size;
-	
+
 	char *content = (char *) malloc((file_len + 2) * sizeof(char)); // + 1 '\0'
 	if (fread(content, 1, file_len, stream) != file_len)
 	{
@@ -69,9 +70,6 @@ static char* get_file_content(const char *file_name)
 
 char* get_file_path_name(const char *dir_name, const char *file_name)
 {
-	if (file_name == NULL) 
-		return NULL;
-	
 #define SEARCH_IN(DIRECTORY) \
 	if (dir_name == NULL)\
 		snprintf(path_file, max_path_len, "%s/%s", DIRECTORY, file_name);\
@@ -87,15 +85,17 @@ char* get_file_path_name(const char *dir_name, const char *file_name)
 	int max_path_len = get_max_path_len();
 
 	char *path_file = (char *) malloc((max_path_len + 1) * sizeof(char));
+
 	// Search by only full path
 	strcpy(path_file, file_name);
+
 	FILE *stream = fopen(path_file, "r");
 	if (stream != NULL)
 	{
 		fclose(stream);
 		return path_file;
 	}
-	
+
 	// Search conf in ~/.xneur
 	if (dir_name == NULL)
 		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, file_name);
@@ -127,19 +127,32 @@ char* get_home_file_path_name(const char *dir_name, const char *file_name)
 	int max_path_len = get_max_path_len();
 
 	char *path_file = (char *) malloc((max_path_len + 1) * sizeof(char));
-	
+
 	if (dir_name == NULL)
 	{
-		snprintf(path_file, max_path_len, "%s/%s", getenv("HOME"), HOME_CONF_DIR); 
-		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); 
+		snprintf(path_file, max_path_len, "%s/%s", getenv("HOME"), HOME_CONF_DIR);
+		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 
 		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, file_name);
 	}
 	else
 	{
-		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, dir_name);
+		char *dir = strdup(dir_name);
+		char *dir_part = strsep(&dir, DIR_SEPARATOR);
+		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, dir_part);
 		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 
+		while (dir != NULL)
+		{
+			path_file = strcat(path_file, DIR_SEPARATOR); 
+			char *dir_part = strsep(&dir, DIR_SEPARATOR);
+			path_file = strcat(path_file, dir_part);
+			mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+		}
+		free(dir_part);
+		
+		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+		
 		snprintf(path_file, max_path_len, "%s/%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, dir_name, file_name);
 	}
 	return path_file;
@@ -147,17 +160,17 @@ char* get_home_file_path_name(const char *dir_name, const char *file_name)
 
 struct _list_char* load_list(const char *dir_name, const char *file_name, int sort_flag)
 {
+	struct _list_char *list = list_char_init();
+	
 	char *file_path_name = get_file_path_name(dir_name, file_name);
 
 	char *content = get_file_content(file_path_name);
 	if (content == NULL)
 	{
 		free(file_path_name);
-		return NULL;
+		return list;
 	}
 
-	struct _list_char *list = list_char_init();
-
 	list->load(list, content);
 
 	free(content);
@@ -172,7 +185,6 @@ struct _list_char* load_list(const char *dir_name, const char *file_name, int so
 int save_list(struct _list_char *list, const char *dir_name, const char *file_name)
 {
 	char *file_path_name = get_home_file_path_name(dir_name, file_name);
-
 	FILE *stream = fopen(file_path_name, "w");
 	if (stream == NULL)
 	{
