@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -32,7 +32,7 @@
 
 static struct _list_char *allocates = NULL;
 
-static const int pointer_len = sizeof(int) * 2 + 3; // + 0x + NULLSYM
+static const int pointer_len = sizeof(void *) * 2 + 3; // + 0x + NULLSYM
 
 void xndebug_init(void)
 {
@@ -59,7 +59,7 @@ void* xndebug_malloc(int len, char *file, int line)
 
 	free(pointer);
 
-	return mem;	
+	return mem;
 }
 
 void xndebug_free(void *mem, char *file, int line)
@@ -90,15 +90,15 @@ char* xndebug_strdup(const char *str, char *file, int line)
 	char *pointer = (char *) malloc(pointer_len * sizeof(char));
 	sprintf(pointer, "%p", mem);
 
-	log_message(TRACE, _("Duping memory pointer %p (at %s:%d)"), str, file, line);
+	log_message(TRACE, _("Duping memory pointer %p (at %s:%d)"), mem, file, line);
 
 	struct _list_char_data *data = allocates->add(allocates, pointer);
 	data->debug_value = line;
-	data->debug_string = file; 
+	data->debug_string = file;
 
 	free(pointer);
 
-	return mem;	
+	return mem;
 }
 
 void* xndebug_realloc(void *mem, int len, char *file, int line)
@@ -114,7 +114,7 @@ void* xndebug_realloc(void *mem, int len, char *file, int line)
 		if (!allocates->exist(allocates, pointer, BY_PLAIN))
 			log_message(ERROR, _("Reallocating invalid memory pointer %p (at %s:%d)"), mem, file, line);
 		else
-			log_message(TRACE, _("(Re)Freeing memory pointer %p (at %s:%d)"), mem, file, line);
+			log_message(TRACE, _("Freeing memory pointer %p (at %s:%d)"), mem, file, line);
 		allocates->rem(allocates, pointer);
 	}
 
@@ -124,11 +124,11 @@ void* xndebug_realloc(void *mem, int len, char *file, int line)
 	{
 		sprintf(pointer, "%p", new_mem);
 
-		log_message(TRACE, _("(Re)Allocating memory pointer %p (at %s:%d)"), new_mem, file, line);
+		log_message(TRACE, _("Allocating memory pointer %p (at %s:%d)"), new_mem, file, line);
 
 		struct _list_char_data *data = allocates->add(allocates, pointer);
 		data->debug_value = line;
-		data->debug_string = file; 
+		data->debug_string = file;
 	}
 
 	free(pointer);
