@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -42,8 +42,8 @@ struct _list_char
 
 	void (*uninit) (struct _list_char *list);
 	struct _list_char_data* (*add) (struct _list_char *list, const char *string);
-	struct _list_char_data* (*add_last) (struct _list_char *list, const char *string);
 	struct _list_char_data* (*find) (struct _list_char *list, const char *string, int mode);
+	struct _list_char_data* (*find_alike) (struct _list_char *list, const char *string);
 	struct _list_char* (*clone) (struct _list_char *list);
 	void (*load)  (struct _list_char *list, char *content);
 	void (*save)  (struct _list_char *list, FILE *stream);
