@@ -13,13 +13,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
  
 #ifndef _REGEXP_H_
 #define _REGEXP_H_
 
-int check_regexp_match(const char *str, const char *pattern);
+char* check_regexp_match(const char *str, const char *pattern);
 
 #endif /* _REGEXP_H_ */
