@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2008 XNeur Team
+ *  Copyright (C) 2006-2009 XNeur Team
  *
  */
 
@@ -21,6 +21,9 @@
 #  include "config.h"
 #endif
 
+#include <sys/time.h>
+#include <sys/resource.h>
+
 #include <signal.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -31,22 +34,28 @@
 
 #include "xnconfig.h"
 
-#include "xprogram.h"
-#include "xbtable.h"
-#include "xswitchlang.h"
-#include "xOSD.h"
+#include "bind_table.h"
+#include "switchlang.h"
+#include "program.h"
+#include "keymap.h"
 
 #include "types.h"
-#include "utils.h"
 #include "list_char.h"
 #include "log.h"
 #include "colors.h"
 #include "sound.h"
+#include "notify.h"
+#include "plugin.h"
+
+#include "newlang_creation.h"
+
+typedef void (*sg_handler)(int);
 
 struct _xneur_config *xconfig = NULL;
-static struct _xprogram *xprogram = NULL;
+static struct _program *program = NULL;
 
 static int xneur_check_lock = TRUE;
+static int xneur_generate_proto = FALSE;
 
 static void xneur_reload(int status);
 
@@ -112,9 +121,9 @@ static void xneur_load_config(int final)
 	{
 		char *lang_name = xconfig->get_lang_name(xconfig, lang);
 
-		log_message(DEBUG, _("%s dictionary has %d records"), lang_name, xconfig->languages[lang].dicts->data_count);
-		log_message(DEBUG, _("%s proto has %d records"), lang_name, xconfig->languages[lang].protos->data_count);
-		log_message(DEBUG, _("%s big proto has %d records"), lang_name, xconfig->languages[lang].big_protos->data_count);
+		log_message(DEBUG, _("%s dictionary has %d records"), lang_name, xconfig->languages[lang].dict->data_count);
+		log_message(DEBUG, _("%s proto has %d records"), lang_name, xconfig->languages[lang].proto->data_count);
+		log_message(DEBUG, _("%s big proto has %d records"), lang_name, xconfig->languages[lang].big_proto->data_count);
 		log_message(DEBUG, _("%s regexp has %d records"), lang_name, xconfig->languages[lang].regexp->data_count);
 	}
 
@@ -122,7 +131,6 @@ static void xneur_load_config(int final)
 
 	log_message(LOG, _("Default group for all new windows set to %d"), xconfig->default_group);
 	log_message(LOG, _("Manual mode set to %s"), xconfig->get_bool_name(xconfig->is_manual_mode(xconfig)));
-	log_message(LOG, _("Mouse processing mode set to %s"), xconfig->get_bool_name(xconfig->grab_mouse));
 	log_message(LOG, _("Education mode set to %s"), xconfig->get_bool_name(xconfig->educate));
 	log_message(LOG, _("Layout remember mode set to %s"), xconfig->get_bool_name(xconfig->remember_layout));
 	log_message(LOG, _("Save selection mode set to %s"), xconfig->get_bool_name(xconfig->save_selection));
@@ -131,9 +139,15 @@ static void xneur_load_config(int final)
 	log_message(LOG, _("Ignore keyboard layout for abbreviations mode set to %s"), xconfig->get_bool_name(xconfig->abbr_ignore_layout));
 	log_message(LOG, _("Correct of iNCIDENTAL CapsLock mode set to %s"), xconfig->get_bool_name(xconfig->correct_incidental_caps));
 	log_message(LOG, _("Correct of two CApital letter mode set to %s"), xconfig->get_bool_name(xconfig->correct_two_capital_letter));
+	log_message(LOG, _("Correct of spaces with punctuation mode set to %s"), xconfig->get_bool_name(xconfig->correct_space_with_punctuation));
+	log_message(LOG, _("Disable CapsLock use mode set to %s"), xconfig->get_bool_name(xconfig->disable_capslock));
 	log_message(LOG, _("Flush internal buffer when pressed Enter or Tab mode set to %s"), xconfig->get_bool_name(xconfig->flush_buffer_when_press_enter));
 	log_message(LOG, _("Don't process word when pressed Enter or Tab mode set to %s"), xconfig->get_bool_name(xconfig->dont_process_when_press_enter));
 	log_message(LOG, _("Show OSD mode set to %s"), xconfig->get_bool_name(xconfig->show_osd));
+	log_message(LOG, _("Show popup messages mode set to %s"), xconfig->get_bool_name(xconfig->show_popup));
+	log_message(LOG, _("Check language on input process mode set to %s"), xconfig->get_bool_name(xconfig->check_lang_on_process));
+	log_message(LOG, _("Pattern minig and recognition (autocomplementation) mode set to %s"), xconfig->get_bool_name(xconfig->autocomplementation));
+	log_message(LOG, _("Add space after autocomplementation mode set to %s"), xconfig->get_bool_name(xconfig->add_space_after_autocomplementation));
 }
 
 static void xneur_set_lock(void)
@@ -149,10 +163,13 @@ static void xneur_set_lock(void)
 	}
 
 	int process_id = getpid();
-
 	xconfig->set_pid(xconfig, process_id);
 
-	log_message(DEBUG, _(PACKAGE " pid is %d"), process_id);
+	int priority = getpriority(PRIO_PROCESS, process_id);
+
+	setpriority(PRIO_PROCESS, process_id, -19);
+	priority = getpriority(PRIO_PROCESS, process_id);
+	log_message(DEBUG, _(PACKAGE" pid is %d with nice %d"), process_id, priority);
 }
 
 static void xneur_cleanup(void)
@@ -160,8 +177,11 @@ static void xneur_cleanup(void)
 	sound_uninit();
 	log_message(DEBUG, _("Current sound data is freed"));
 
-	if (xprogram != NULL)
-		xprogram->uninit(xprogram);
+	unbind_user_actions();
+	log_message(DEBUG, _("Current user keybinds is freed"));
+	
+	if (program != NULL)
+		program->uninit(program);
 
 	log_message(DEBUG, _("Current program info is freed"));
 
@@ -182,8 +202,10 @@ static void xneur_terminate(int status)
 	if (status){}
 
 	log_message(DEBUG, _("Caught SIGTERM/SIGINT, terminating"));
-	osd_show(xconfig->osds[OSD_XNEUR_STOP].file);
+	show_notify(NOTIFY_XNEUR_STOP, NULL);
 	sleep(1);
+
+	program->plugin->xneur_stop(program->plugin);
 	
 	xneur_cleanup();
 
@@ -192,11 +214,13 @@ static void xneur_terminate(int status)
 
 static void xneur_reload(int status)
 {
-	status = status; // To prevent warnings
+	if (status){}
+
 	log_message(LOG, _("Caught SIGHUP, reloading configuration file"));
-	osd_show(xconfig->osds[OSD_XNEUR_RELOAD].file);
-	
+	show_notify(NOTIFY_XNEUR_RELOAD, NULL);
+
 	sound_uninit();
+	unbind_user_actions();
 
 	if (xconfig != NULL)
 		xconfig->uninit(xconfig);
@@ -211,6 +235,8 @@ static void xneur_reload(int status)
 	xneur_load_config(TRUE);
 	xneur_init();
 	sound_init();
+
+	program->plugin->xneur_reload(program->plugin);
 }
 
 static void xneur_usage(void)
@@ -223,6 +249,7 @@ static void xneur_usage(void)
 	printf("  -h, --help              This help!\n");
 	printf("  -a, --about             About for " PACKAGE "\n");
 	printf("  -f, --force             Skip check for other instances of " PACKAGE " runned\n");
+	printf("  -g, --generate          Generate proto for new language. THIS OPTION FOR DEVELOPERS ONLY!\n");
 }
 
 static void xneur_version(void)
@@ -250,6 +277,7 @@ static void xneur_get_options(int argc, char *argv[])
 			{ "help",		no_argument,	NULL,	'h' },
 			{ "about",		no_argument,	NULL,	'a' },
 			{ "force",		no_argument,	NULL,	'f' },
+			{ "generate",	no_argument,	NULL,	'g' },
 			{ NULL,			0,		NULL,	0 }
 	};
 
@@ -276,6 +304,12 @@ static void xneur_get_options(int argc, char *argv[])
 				opted = FALSE;
 				break;
 			}
+			case 'g':
+			{
+				xneur_generate_proto = TRUE;
+				opted = FALSE;
+				break;
+			}
 			case '?':
 			case 'h':
 			{
@@ -298,6 +332,19 @@ static void xneur_reklama(void)
 	printf("\n");
 }
 
+static void xneur_trap(int sig, sg_handler handler)
+{
+	struct sigaction sa;
+	bzero(&sa, sizeof(sa));
+	sa.sa_handler = handler;
+
+	if (sigaction(sig, &sa, NULL) == -1)
+	{
+		log_message(ERROR, _("Can't trap signal"));
+		exit(EXIT_FAILURE);
+	}
+}
+
 int main(int argc, char *argv[])
 {
 #ifdef ENABLE_NLS
@@ -306,7 +353,8 @@ int main(int argc, char *argv[])
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 #endif
-	
+
+
 	xneur_reklama();
 
 	xneur_get_options(argc, argv);
@@ -321,8 +369,8 @@ int main(int argc, char *argv[])
 	xneur_set_lock();
 	xneur_load_config(FALSE);
 
-	xprogram = xprogram_init();
-	if (xprogram == NULL)
+	program = program_init();
+	if (program == NULL)
 	{
 		log_message(ERROR, _("Failed to init program structure"));
 		xconfig->set_pid(xconfig, 0);
@@ -334,13 +382,18 @@ int main(int argc, char *argv[])
 	xneur_init();
 
 	log_message(DEBUG, _("Init program structure complete"));
-	osd_show(xconfig->osds[OSD_XNEUR_START].file);
-	
-	xntrap(SIGTERM, xneur_terminate);
-	xntrap(SIGINT, xneur_terminate);
-	xntrap(SIGHUP, xneur_reload);
+	show_notify(NOTIFY_XNEUR_START, NULL);
 
-	xprogram->process_input(xprogram);
+	xneur_trap(SIGTERM, xneur_terminate);
+	xneur_trap(SIGINT, xneur_terminate);
+	xneur_trap(SIGHUP, xneur_reload);
+
+	program->plugin->xneur_start(program->plugin);
+			
+	if (xneur_generate_proto)
+		generate_protos();
+	else
+		program->process_input(program);
 
 	xneur_cleanup();
 
